print("  ==============================================================")
print("  |     Python3 program for testing communication towards      |")
print("  |                                                            |")
print("  |     WVC-600 inverter via RF-interface on 433.400 MHz       |")
print("  |                          based on                          |")
print("  |    USB-serial-module and HC-12 data-transceiver-module     |")
print("  |                                                            |")
print("  | Author: Jaap van den Berg  Date: 31-DEC-2021  Version: 0.1 |")
print("  ==============================================================")

# ---------------------------------------------input-parameters-----------------------------------------
# Select correct COM-port!!

# if Linux, un-comment following line
# computer_COM_port="/dev/ttyUSB0"

# if Windows, un-comment following line and enter correct COM-port (displayed by Windows configuration)
computer_COM_port="COM3"

# Enter correct WVC-600 inverter-id!! (see sticker on inverter)           
wvc600_id = "12345678"
# ------------------------------------------------------------------------------------------------------

print()
print("Program will send a hexadecimal request-code via com-port",computer_COM_port)
print("to WVC-600 inverter with id",wvc600_id)
print("(correct this in -----input-parameters----- of the Python-program if necessary)")

print()
input("Press Enter to continue")

# Import of necessary Python-modules
import serial

# Construction of command to be sent to the WVC-600 inverter
command_firstpart_hex = "f20000000000000000"
command_lastpart_hex = "65"
command_total_hex = command_firstpart_hex + wvc600_id + command_lastpart_hex
command_to_wvc600 = bytes.fromhex(command_total_hex)

print()
print("Hexadecimal request-code sent to the WVC-600-inverter with id",wvc600_id,":")
print(command_total_hex)

# Open serial communication port at 9600 bits/sec (default 8 data-bits, no parity)
serial_comm_port = serial.Serial(port=computer_COM_port, baudrate=9600, timeout=1, write_timeout=1)
serial_comm_port.write(command_to_wvc600)
wvc600_response = serial_comm_port.read(28)
serial_comm_port.close()

if len(wvc600_response) == 28:
   wvc600_hex_output = wvc600_response.hex()
   print()
   print("WVC-600 inverter-response (hexadecimal): ")
   print(wvc600_hex_output)
   print()
   input("Press Enter to continue")
   print()
   print("WVC-600 inverter measurement-data decoded from response")
   Vdc_hex = wvc600_hex_output[28:32]
   Vdc = int(Vdc_hex,16)/100
   print("Vdc  =",Vdc,"V")
   Vac_hex = wvc600_hex_output[32:36]
   Vac = int(Vac_hex,16)/100
   print("Vac  =",Vac,"V")
   Idc_hex = wvc600_hex_output[36:40]
   Idc = int(Idc_hex,16)/100
   print("Idc  =",Idc,"A")
   Etot_hex = wvc600_hex_output[40:44]
   Etot = int(Etot_hex,16)/100
   print("Etot =",Etot,"kWh")
   Pout_hex = wvc600_hex_output[48:52]
   Pout = int(Pout_hex,16)/10
   print("Pout =",Pout,"W")
   Temp_hex = wvc600_hex_output[52:56]
   Temp = int(Temp_hex,16)/100
   print("Temp =",Temp,"gr.C")
else:
   print()
   print("WVC-600 inverter did not respond correctly!")
   print("Possible causes:")
   print("--Wrong WVC-600 inverter-id used")
   print("--Wrong com-port used")
   print("--Malfunction in data-communication system")
   print("--Insufficient power from the solar panels")

print()
input("Press Enter to continue")
